from .spells import Spell, create_spell
from .spells_a import *
from .spells_b import *
from .spells_c import *
from .spells_d import *
from .spells_e import *
from .spells_f import *
from .spells_g import *
from .spells_h import *
from .spells_i import *
from .spells_j import *
from .spells_k import *
from .spells_l import *
from .spells_m import *
from .spells_n import *
from .spells_o import *
from .spells_p import *
from .spells_q import *
from .spells_r import *
from .spells_s import *
from .spells_t import *
from .spells_u import *
from .spells_v import *
from .spells_w import *
from .spells_x import *
from .spells_y import *
from .spells_z import *
